/* Nitrous Texture Tweaker Comment Header
-------------------------------------------------------------------------------------------------------------------------------------------
---- Nitrous Texture Tweaker V1.0 -  RELEASE
-------------------------------------------------------------------------------------------------------------------------------------------
---- Nitrous Texture Tweaker :  Nitrous Texturelimit Settings UI
---- Created:  October 1, 2012
---- Modified: October 1, 2012

HISTORY:
---- Version:  V1.0.1 - Hotfix Release, DC20121001
----	Fixes missing procedural texture code

---- Version:  V1.0.0 - Initial Release, DC20121001

LIMITATIONS:
----	Brute Force material loop, hence toggling map display mode may take longer as really required
----	Map display modes might fail on some material types, like "Shell Material" etc..

INSTALLATION:
----    Open this Script in Maxscript Editor and press "Evaluate" ( Ctrl-E )
----    You will find it the "Customize User Interface Dialog" in the "Spacefrog Tools" - category
----	Simply map "Nitrous: Viewport Texture Tweaker" to a keyboard shortcut, menu entry or button as usual

AUTHOR:
----	Josef Wienerroither aka "spacefrog"  www.frogsinspace.at  | spacefrog@chello.at
----	Parts inspired by Kelly Michels's ( Autodesk ) "Nitrous Set Texture Size Limit" script
-------------------------------------------------------------------------------------------------------------------------------------------
*/
macroScript NitrousTextureTweaker category:"Spacefrog Tools"	toolTip:"Nitrous: Viewport Texture Tweaker"
(
	global SF_Nitrous_TextureTweaker

	try ( DestroyDialog SF_Nitrous_TextureTweaker ) catch()

	local AppVersion = "1.0.1"
	local AppVersionString = " V" +appVersion;

	local AppTitle = "Nitrous Texture Tweaker";
	local AppCredits ="Author: Josef \"spacefrog\" Wienerroither 2012";
	
	local MaxVersionMajor = (maxversion())[1]	
	
	local MaxLanguageIDs = #( 0x09,0x07,0x0C ); -- not implemented:  ....,0x04,0x11,0x12 );
	local MaxLanguageIndex = findItem MaxLanguageIDs ((sysinfo.GetMaxLanguage())[1])
	local uitxtViewportConfig  = #("Viewport Configuration", "Ansichtsfenster konfigurieren", "Configuration fentre","CHINESE", "JAPANESE", "KOREAN" )
		
		
	rollout roNitrousTweaker (AppTitle+AppVersionString) width:260
	(
		local oldGFXTexSize, oldBGTextureSizeLimit, oldBGTextureSizeLimitEnabled, oldTextureSizeLimit, oldTextureSizeLimitEnabled
		local GFXTexSizeIsDirty=false
		local proceduralResolutionToSet=512
		local nsINISettings
		local MemoryHeadText = "Nitrous Memory Usage: "

		
		struct nsINISettingsStruct
		(
			nsINISectionName="Nitrous",
			maxIniFileName = getMAXIniFile(),
			GFXTexSize,
			TextureSizeLimitEnabled,
			TextureSizeLimit,
			BGTextureSizeLimitEnabled,
			BGTextureSizeLimit,
			
			--forward declarations
			function Read=(),
			
			
			function _init=
			(
				Read()	
			),
			
			-- reads values from the ini file 
			function Read=
			(
				GFXTexSize =  (getINISetting maxIniFileName nsINISectionName "GFXTexSize") as integer
				TextureSizeLimitEnabled =  getINISetting maxIniFileName nsINISectionName "ViewportTextureSizeLimitEnabled"
				TextureSizeLimit =  (getINISetting maxIniFileName nsINISectionName "ViewportTextureSizeLimit") as integer 
				BGTextureSizeLimitEnabled =  getINISetting maxIniFileName nsINISectionName "ViewportBackgroundTextureSizeLimitEnabled"
				BGTextureSizeLimit =  (getINISetting maxIniFileName nsINISectionName "ViewportBackgroundTextureSizeLimit") as integer

				if(GFXTexSize == 0 ) then  GFXTexSize=512
				
				if ( TextureSizeLimitEnabled == "1" ) then TextureSizeLimitEnabled = true else  TextureSizeLimitEnabled= false
				
				if( TextureSizeLimit == 0) then TextureSizeLimit=512
				
				if ( BGTextureSizeLimitEnabled == "1" ) then BGTextureSizeLimitEnabled = true else  BGTextureSizeLimitEnabled= false
				if( BGTextureSizeLimit == 0 ) then BGTextureSizeLimit=1024
			),
			
			-- applies & writes values out directly to 3sdmax.ini - this prevents values getting lost because of Max crashin
			function Write=
			(
				setINISetting maxIniFileName nsINISectionName "GFXTexSize" (GFXTexSize as string)
				
				if ( TextureSizeLimitEnabled == true ) then boolValString = "1" else  boolValString="0"
				setINISetting maxIniFileName nsINISectionName "ViewportTextureSizeLimitEnabled" boolValString
				
				setINISetting maxIniFileName nsINISectionName "ViewportTextureSizeLimit" (TextureSizeLimit as string )
				
				if ( BGTextureSizeLimitEnabled == true ) then boolValString = "1" else  boolValString="0"
				setINISetting maxIniFileName nsINISectionName "ViewportBackgroundTextureSizeLimitEnabled" boolValString
				
				setINISetting maxIniFileName nsINISectionName "ViewportBackgroundTextureSizeLimit" (BGTextureSizeLimit as string)

				--- how to set "GFXTexSize" ?	-> done in the rollout code via UIAccessor to control the viewport config dialog										
				nitrousgraphicsmanager.SetTextureSizeLimit TextureSizeLimit TextureSizeLimitEnabled
				nitrousgraphicsmanager.SetBackgroundTextureSizeLimit BGTextureSizeLimit BGTextureSizeLimitEnabled
			),
			
			_result=_init()
		)	
		

		
		Group "Resolution Limits"
		(
			spinner spTextureSizeLimit "Textures*" type:#integer range:[4,8192,512] align:#right fieldwidth:40 offset:[64,1] across:2
			checkbutton cbTextureSizeLimitEnable "Enable" align:#right height:18

			spinner spBGTextureSizeLimit "Background Textures" type:#integer range:[4,8192,1024] align:#right offset:[64,1] fieldwidth:40 across:2
			checkbutton cbBGTextureSizeLimitEnable "Enable" align:#right height:18

			spinner spGFXTexSize "Procedural Maps" type:#integer range:[128,4096,512] align:#right fieldwidth:40 across:2 offset:[64,1] enabled:true
			checkbutton cbDummy01 "Enable" align:#right height:18 visible:false				

			
			label laTextureSizeHint "*Applies to Shaded Material Display Mode only" offset:[0,4]
		)
		
		Group "Material Display Utilities"
		(
			label laSelectedTo "Selected Objects To..." align:#left across:2
			label laAllTo "All Objects To..." align:#left offset:[6,0]
			
			button btShadedSelected "Shaded" width:46 align:#left height:18 across:4
			button btShadedMapsSelected "Shade+Map" width:64 align:#left  height:18 offset:[-9,0]
			button btShadedAll "Shaded" width:46 align:#left   height:18 offset:[6,0]
			button btShadedMapsAll "Shade+Map" width:64 align:#left  height:18 offset:[-4,0]


			button btRealisticSelected "Realistic" width:46 align:#left  height:18  across:4  offset:[0,-2]				
			button btRealisticMapsSelected "Real+Map" align:#left  width:64 height:18 offset:[-9,-2]
			button btRealisticAll "Realistic" width:46 align:#left  height:18 offset:[6,-2]
			button btRealisticMapsAll "Real+Map" width:64 align:#left  height:18 offset:[-4,-2]
		)
		
		label laDummy01 "" height:0 offset:[0,-12] visible:false
		Group "" 
		(
			label laMemoryState "Nitrous Memory Usage: ( 3ds Max 2013+ only )" align:#center width:(roNitrousTweaker.width-16) style_sunkenedge:false offset:[2,-9] enabled:( MaxVersionMajor>=14900 )
		)
				
		button btRevert "Revert" across:4 align:#right offset:[-6,0] width:54
		button btDefaults "Defaults" align:#right offset:[-6,0] width:54
		button btApply "Apply" align:#right width:49 offset:[8,0]
		button btExit "Exit" align:#right width:48  offset:[2,0]

		label laCopyright01 appCredits align:#left
		
		
		function GetNodeMaterials nodes:$objects=
		(
			for obj in nodes where (obj.material != undefined ) collect obj.material				
		)			
		
		function RefreshMemoryDisplay=
		(
			if( MaxVersionMajor>=14900 ) then
				laMemoryState.text = MemoryHeadText + ( (nitrousgraphicsmanager.GetSystemMemoryUsed() as float)/1048576 ) as string + " MBytes"
		)
		
		function RefreshMapDisplay mats=
		(
			for mat in mats where mat.showInViewport == true do
			(
				mat.showinviewport=mapDisplay=false
				mat.showinviewport=mapDisplay=true
			)
		)
		
		function SetMaterialDisplay mats hwDisplay mapDisplay=
		(
			for mat in mats do 
			( 
				--format "HW:%\t\tM:%\t\tMat: %\n" hwDisplay mapDisplay mat 
				
				enableHardwareMaterial mat hwDisplay 
				
				if(hwDisplay==true) then 
					showHWTextureMap mat mapDisplay
				else
				(
					showTextureMap mat mapDisplay
					mat.showinviewport=mapDisplay -- enable mnap display accordingly
				)
			)
			redrawViews() --- required to generate mapping coordinates where not existant
		)

		function CaptureVPTProceduralResolution=
		(
			local notFound=tabNotFound=edNotFound=true
			local tabIndex=3 --- 3ds Max 2012
			local TCM_SETCURFOCUS=0x1330, WM_CHAR = 0x0102,	VK_RETURN = 0x000D
			
			if( MaxVersionMajor >= 14900 ) then tabindex = 4 --- 3ds Max2013 and up
		

			
			for hDlg in UIAccessor.GetPopupDialogs() while notFound do
			(
				-- let's hope VP dialog's title doesn't change
				if((UIAccessor.GetWindowText hDlg )==  uitxtViewportConfig[MaxLanguageIndex] ) then
				(
				    notFound=false
					-- get the Tab-Control handle

					for hChild in (UIAccessor.GetChildWindows hDlg) where ( UIAccessor.GetWindowClassName hChild == "SysTabControl32" ) while tabNotFound do
					(		
						tabNotFound = false;

						UIAccessor.SendMessage hChild TCM_SETCURFOCUS tabIndex 0
						
						--- editcontol text is always "ProceduralMapsDisplayResolution" , independend of the Max localization !!!
						
						edCtrlParent = windows.getChildHWND hDlg "ProceduralMapsDisplayResolution"
						
						if( edCtrlParent != undefined ) then
						(
							--UIAccessor.SetWindowText edCtrlParent[1] ( proceduralResolutionToSet as string + "\n" )
							for edCtrl  in (windows.GetChildrenHWND edCtrlParent[1]) where ( UIAccessor.GetWindowClassName edCtrl[1]== "Edit" ) while edNotFound do
							(
								edNotFound=false
								
								UIAccessor.SetWindowText  (edCtrl[1]) (proceduralResolutionToSet as string)
								windows.sendMessage (edCtrl[1]) WM_CHAR VK_RETURN 0 -- press ENTER key								
								
								UIAccessor.SendMessageID hDlg #IDOK
							)
						)						
					)
					if( edNotFound== true ) then UIAccessor.SendMessageID hDlg #IDCANCEL
				)
			)
			DialogMonitorOPS.Enabled = false
			DialogMonitorOPS.unRegisterNotification id:#ProcResCatcher

			success=NOT edNotFound;
		)
		
		function SetProceduralResolution res=		
		(
			proceduralResolutionToSet=res
			
			DialogMonitorOPS.unRegisterNotification id:#ProcResCatcher


			try
			(
				DialogMonitorOPS.RegisterNotification CaptureVPTProceduralResolution id:#ProcResCatcher

				DialogMonitorOPS.Interactive=false
				DialogMonitorOPS.Enabled = true

				max vptconfig
			)
			catch
			(
				DialogMonitorOPS.Enabled = false
				DialogMonitorOPS.unRegisterNotification id:#ProcResCatcher
			)
			DialogMonitorOPS.Enabled = false
			DialogMonitorOPS.unRegisterNotification id:#ProcResCatcher		
		)
		

		
		function LoadValues=
		(
			oldGFXTexSize=spGFXTexSize.value=nsINISettings.GFXTexSize
			oldTextureSizeLimit=spTextureSizeLimit.value=nsINISettings.TextureSizeLimit
			oldTextureSizeLimitEnabled=cbTextureSizeLimitEnable.checked=nsINISettings.TextureSizeLimitEnabled
			oldBGTextureSizeLimit=spBGTextureSizeLimit.value=nsINISettings.BGTextureSizeLimit
			oldBGTextureSizeLimitEnabled=cbBGTextureSizeLimitEnable.checked=nsINISettings.BGTextureSizeLimitEnabled							
		)
		
		function SaveValues=
		(
			nsINISettings.GFXTexSize=spGFXTexSize.value
			nsINISettings.TextureSizeLimit=spTextureSizeLimit.value
			nsINISettings.TextureSizeLimitEnabled=cbTextureSizeLimitEnable.checked
			nsINISettings.BGTextureSizeLimit=spBGTextureSizeLimit.value
			nsINISettings.BGTextureSizeLimitEnabled=cbBGTextureSizeLimitEnable.checked							
		)
		
		on roNitrousTweaker open do
		(
			nsINISettings	= nsINISettingsStruct()

			LoadValues()
			RefreshMemoryDisplay()
		)
		
		on spGFXTexSize changed newVal do
		(
			GFXTexSizeIsDirty=true
		)
		
		on btRevert pressed do
		(
			spGFXTexSize.value=oldGFXTexSize
			spTextureSizeLimit.value=oldTextureSizeLimit
			cbTextureSizeLimitEnable.checked=oldTextureSizeLimitEnabled
			spBGTextureSizeLimit.value=oldBGTextureSizeLimit
			cbBGTextureSizeLimitEnable.checked=oldBGTextureSizeLimitEnabled
		)

		on btDefaults pressed do
		(
			spGFXTexSize.value=512
			spTextureSizeLimit.value=1024
			cbTextureSizeLimitEnable.checked=false
			spBGTextureSizeLimit.value=1024
			cbBGTextureSizeLimitEnable.checked=false
		)
		
		on btApply pressed do
		(
			SaveValues()
			nsINISettings.Write()
			
			if( GFXTexSizeIsDirty==true ) then
			(
				GFXTexSizeIsDirty = false
				SetProceduralResolution spGFXTexSize.value
			)
			
			RefreshMapDisplay (GetNodeMaterials())
			RefreshMemoryDisplay()
		)
		
		on btExit pressed do
		(
			DestroyDialog roNitrousTweaker
		)
		
		on btShadedSelected  pressed do   	(  SetMaterialDisplay (GetNodeMaterials nodes:$selection ) false false )
		on btRealisticSelected pressed do 	(  SetMaterialDisplay (GetNodeMaterials nodes:$selection ) true  false )
		on btShadedAll pressed do		  	(  SetMaterialDisplay (GetNodeMaterials()) false false  )
		on btRealisticAll pressed do        (  SetMaterialDisplay (GetNodeMaterials()) true  false  )
										
		on btShadedMapsSelected pressed do      (  SetMaterialDisplay (GetNodeMaterials nodes:$selection )  false true ) 
		on btRealisticMapsSelected  pressed do  (  SetMaterialDisplay (GetNodeMaterials nodes:$selection )  true  true )
		on btShadedMapsAll pressed do       	(  SetMaterialDisplay (GetNodeMaterials())  false true )
		on btRealisticMapsAll pressed do        (  SetMaterialDisplay (GetNodeMaterials())  true  true )

	)
		
	if( MaxVersionMajor<14000 OR  nitrousgraphicsmanager.isEnabled == false ) then
	(
		MessageBox "Nitrous Viewport Driver not available.\nTerminating..." title:(AppTitle+AppVersionString)
		SF_Nitrous_TextureTweaker = undefined		
		_result=AppTitle +AppVersionString+ " launch error..."
	)	
	else
	(
		_result = CreateDialog roNitrousTweaker  style:#(#style_resizing,#style_titlebar,#style_sysmenu,#style_minimizebox) lockWidth:true

		if( _result ==  true ) then
		(
			SF_Nitrous_TextureTweaker = roNitrousTweaker
			_result=ok
		)
	)
	_result
)